package com.redxun.knowledge.album.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.knowledge.album.entity.dao.AlbumCatalogue;
import com.redxun.knowledge.album.entity.dao.AlbumCatalogueKnowledge;
import com.redxun.knowledge.album.entity.dao.AlbumInfo;
import com.redxun.knowledge.album.entity.dao.AlbumVersion;
import com.redxun.knowledge.album.entity.vo.AlbumCataloguePcVo;
import com.redxun.knowledge.album.entity.vo.AlbumKnowledgeVo;
import com.redxun.knowledge.album.mapper.AlbumCatalogueKnowledgeMapper;
import com.redxun.knowledge.album.mapper.AlbumCatalogueMapper;
import com.redxun.knowledge.album.mapper.AlbumInfoMapper;
import com.redxun.knowledge.album.mapper.AlbumVersionMapper;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.service.KnowledgeCategoryServiceImpl;
import com.redxun.knowledge.service.KnowledgeServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * [业务--专辑版本表]业务服务类
 */
@Service
public class AlbumVersionServiceImpl extends SuperServiceImpl<AlbumVersionMapper, AlbumVersion> implements BaseService<AlbumVersion> {

    @Resource
    private AlbumVersionMapper albumVersionMapper;

    @Autowired
    private UserService userService;

    @Autowired
    private AlbumInfoMapper albumInfoMapper;

    @Autowired
    private AlbumCatalogueMapper albumCatalogueMapper;

    @Autowired
    private KnowledgeServiceImpl knowledgeService;

    @Autowired
    private AlbumCatalogueKnowledgeMapper albumCatalogueKnowledgeMapper;

    @Autowired
    private KnowledgeCategoryServiceImpl knowledgeCategoryService;

    @Override
    public BaseDao<AlbumVersion> getRepository() {
        return albumVersionMapper;
    }

    /**
     * PC端根据知识专辑ID和版本ID获取知识专辑目录及目录下知识
     *
     * @param albumId
     * @param versionId
     * @return
     */
    public List<AlbumCataloguePcVo> catalogue(String albumId, String versionId) {
        //判断登录人是否是专辑所属人(是 查看审核中的版本(没有审核中的,查看已发布的) 不是 查看已发布的版本)
        String userId = userService.queryLoginUser();
        AlbumInfo albumInfo = albumInfoMapper.selectById(albumId);
        if (albumInfo.getOwnerId().equals(userId)) {
            //查询版本信息(审核中)
            AlbumVersion albumVersion1 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", albumId).eq("STATE", 1));
            //查询版本信息(驳回)
            AlbumVersion albumVersion4 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", albumId).eq("STATE", 4));
            if (albumVersion1 != null) {
                List<AlbumCataloguePcVo> catalogue = getCatalogue(albumInfo.getOwnerId(), albumVersion1.getPkId());
                return catalogue;
            } else if (albumVersion4 != null) {
                List<AlbumCataloguePcVo> catalogue = getCatalogue(albumInfo.getOwnerId(), albumVersion4.getPkId());
                return catalogue;
            } else {
                //生效中的专辑
                AlbumVersion albumVersion2 = albumVersionMapper.
                        selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", albumId).eq("STATE", 2));
                List<AlbumCataloguePcVo> catalogue = getCatalogue(albumInfo.getOwnerId(), albumVersion2.getPkId());
                return catalogue;
            }
        } else {
            //生效中的专辑
            AlbumVersion albumVersion2 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", albumId).eq("STATE", 2));
            List<AlbumCataloguePcVo> catalogue = getCatalogue(albumInfo.getOwnerId(), albumVersion2.getPkId());
            return catalogue;
        }
    }

    /**
     * PC端查询目录信息
     */
    private List<AlbumCataloguePcVo> getCatalogue(String ownerId, String versionId) {
        String userId = userService.queryLoginUser();
        List<AlbumCatalogue> albumCatalogueList = albumCatalogueMapper.selectList(new LambdaQueryWrapper<AlbumCatalogue>().
                eq(AlbumCatalogue::getVersionId, versionId).orderByAsc(AlbumCatalogue::getSort));
        List<AlbumCataloguePcVo> albumCataloguePcVos = albumCatalogueList.parallelStream().map(e -> {
            AlbumCataloguePcVo albumCataloguePcVo = new AlbumCataloguePcVo();
            BeanUtils.copyProperties(e, albumCataloguePcVo);
            //找到对应的版本
            AlbumVersion albumVersion = albumVersionMapper.selectById(versionId);
            //动作标记
            List<String> actionFlag = new ArrayList<>();
            if (ownerId.equals(userId) && albumVersion.getState() != 1) {
                actionFlag.add("edit");
                actionFlag.add("delete");
            }
            albumCataloguePcVo.setActionFlag(actionFlag);
            return albumCataloguePcVo;
        }).collect(Collectors.toList());
        //目录下的知识内容
        albumCataloguePcVos.forEach(albumCatalogueVo -> {
            //查询对应的知识Id
            List<AlbumCatalogueKnowledge> albumCatalogueKnowledgeList = albumCatalogueKnowledgeMapper.
                    findAllAlbumCatalogueKnowledgeList(albumCatalogueVo.getPkId());
            //查询对应的知识
            List<AlbumKnowledgeVo> albumKnowledgeVoList = albumCatalogueKnowledgeList.parallelStream().map(albumCatalogueKnowledge -> {
                AlbumKnowledgeVo albumKnowledgeVo = new AlbumKnowledgeVo();
                Knowledge knowledge = knowledgeService.get(albumCatalogueKnowledge.getKnowledgeId());
                if (knowledge != null) {
                    albumKnowledgeVo.setKnowledgeId(knowledge.getPkId());
                    //查询分类信息
                    KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(knowledge.getCategoryId());
                    String category = knowledgeCategoryAdminVo.getName();
                    if (knowledgeCategoryAdminVo.getParents() != null) {
                        category = knowledgeCategoryAdminVo.getParents().getName() + "|" + category;
                        if (knowledgeCategoryAdminVo.getParents().getParents() != null) {
                            category = knowledgeCategoryAdminVo.getParents().getParents().getName() + "|" +
                                    knowledgeCategoryAdminVo.getParents().getName() + "|" + category;
                        }
                    }
                    albumKnowledgeVo.setCategory(category);
                    albumKnowledgeVo.setTitle(knowledge.getTitles());
                    albumKnowledgeVo.setAuthor(knowledge.getAuthor());
                    albumKnowledgeVo.setAuthorHead((String) userService.querySexAndPhoto(knowledge.getCreateBy()).get("photo"));
                    albumKnowledgeVo.setCreateTime(knowledge.getCreateTime());
                }
                return albumKnowledgeVo;
            }).collect(Collectors.toList());
            albumCatalogueVo.setContents(albumKnowledgeVoList);
        });
        return albumCataloguePcVos;
    }
}
