package com.redxun.knowledge.analysis.mapper;


import com.redxun.knowledge.analysis.entity.dao.KnowledgeDao;
import com.redxun.knowledge.analysis.entity.vo.KnowledgeSumAndYelpSumListVo;
import com.redxun.knowledge.analysis.entity.vo.SearchParticipleWordCloudVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 文件名: CommonMapper
 * 作者: zizg
 * 时间: 2023/3/17
 * 描述:
 * 修改人:
 * 修改时间:
 * 修改内容:
 */
@Mapper
public interface AnalysisCommonMapper {

    /**
     * PC知识仓库统计(审核通过的知识数量)
     *
     * @return
     */
    Integer knowledgeTotalAmount();

    /**
     * PC地图数量统计
     *
     * @return
     */
    Integer mapTotalAmount();

    /**
     * PC专辑数量统计
     *
     * @return
     */
    Integer albumTotalAmount();


    /**
     * 根据类型查看知识数量(1 文档知识 2 维基知识)
     *
     * @param type
     * @return
     */
    Integer knowledgeTotalByType(Integer type);

    /**
     * 获取搜索服务分词的词云数据
     *
     * @param tops
     * @return
     */
    List<SearchParticipleWordCloudVo> searchParticipleWordCloud(Integer tops);

    /**
     * 知识类型访问量分布
     *
     * @param firstDay
     * @param lastDay
     * @return
     */
    List<String> knowledgeTypeVisitProportion(@Param("firstDay") String firstDay, @Param("lastDay") String lastDay);

    /**
     * 获取全部知识一级分类id集合
     *
     * @return
     */
    List<String> level1Knowledge();

    /**
     * 获取一级知识分类下的全部知识数量
     *
     * @param categoryId
     * @param firstDay
     * @param lastDay
     * @return
     */
    List<Integer> level1KnowledgeHistogram(@Param("categoryId") String categoryId, @Param("firstDay") String firstDay, @Param("lastDay") String lastDay);


    String selectCategoryName(String categoryId);

    /**
     * 获取各个公司创建的知识数量
     *
     * @param firstDay
     * @param lastDay
     * @return
     */
    List<KnowledgeDao> organizationKnowledges(@Param("firstDay") String firstDay, @Param("lastDay") String lastDay);

    /**
     * 组织创建知识数量(新)
     *
     * @param firstDay
     * @param lastDay
     * @return
     */
    List<KnowledgeDao> organizationVisit(@Param("firstDay") String firstDay, @Param("lastDay") String lastDay,
                                         @Param("organizationId") String organizationId);

    List<Integer> personKnowledgeValue(@Param("id") String id, @Param("firstDay") String createTime,
                                       @Param("lastDay")String endTime,@Param("round") Integer round);

    /**
     * 当前用户知识上传量
     * @param userId
     * @return
     */
    Integer knowledgeSum(@Param("userId") String userId,@Param("beginTime") String beginTime,@Param("endTime") String endTime);

    /**
     * 当前用户专辑评论量
     * @param userId
     * @return
     */
    Integer yelpSum(@Param("userId") String userId,@Param("beginTime") String beginTime,@Param("endTime") String endTime);

    /**
     * 展示员工知识上传数和专辑评论数列表
     * @return
     */
    List<KnowledgeSumAndYelpSumListVo> knowledgeSumAndYelpSumList(@Param("beginTime") String beginTime, @Param("endTime") String endTime,
                                                                  @Param("pageNo") Integer pageNo, @Param("pageSize")Integer pageSize);
}
