
package com.redxun.knowledge.controller;

import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.entity.JsonPageResult;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.knowledge.entity.dao.KnowledgeAgent;
import com.redxun.knowledge.entity.dto.KnowledgeAgentDto;
import com.redxun.knowledge.entity.vo.KnowledgeAgentVo;
import com.redxun.knowledge.service.BannerServiceImpl;
import com.redxun.knowledge.service.KnowledgeAgentServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/knowledge/knowledgeAgent")
@Api(tags = "业务--知识代理人表")
@ClassDefine(title = "知识代理人", alias = "knowledgeAgentController", path = "/api-knowledge/knowledge/knowledgeAgent", packages = "knowledge", packageName = "知识管理")
public class KnowledgeAgentController extends BaseController<KnowledgeAgent> {

    @Autowired
    BannerServiceImpl bannerService;

    @Autowired
    private KnowledgeAgentServiceImpl knowledgeAgentService;


    @Override
    public KnowledgeAgentServiceImpl getBaseService() {
        return knowledgeAgentService;
    }

    @Override
    public String getComment() {
        return "业务--代理人";
    }

    @ApiOperation("新增代理人")
    @PostMapping("create")
    public JsonResult create(@Valid @RequestBody KnowledgeAgentDto knowledgeAgentDto, BindingResult bindingResult){
        JsonResult jsonResult = JsonPageResult.getSuccess("");
        if (bindingResult.hasErrors()){
            jsonResult.setMessage("参数错误");
            return jsonResult;
        }
        String pkId = null;
        try {
            pkId = knowledgeAgentService.create(knowledgeAgentDto);
            jsonResult.setData(pkId);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setSuccess(false);
        }
        return jsonResult;

    }

    @ApiOperation("查询代理人列表")
    @GetMapping("info")
    public JsonResult info(){
        JsonResult jsonResult = JsonPageResult.getSuccess("");
        KnowledgeAgentVo knowledgeAgentVo = knowledgeAgentService.info();
        return jsonResult.setData(knowledgeAgentVo);
    }


    @ApiOperation("清空代理人")
    @GetMapping("del")
    public JsonResult del(){
        JsonResult jsonResult = JsonPageResult.getSuccess("");
        String pkId = knowledgeAgentService.del();
        return jsonResult.setData(pkId);
    }



}

