
package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.model.JPaasUser;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.BeanUtil;
import com.redxun.common.tool.IdGenerator;
import com.redxun.common.tool.StringUtils;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.dao.KnowledgeCategory;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.mapper.KnowledgeCategoryMapper;
import com.redxun.knowledge.mapper.KnowledgeMapper;
import com.redxun.knowledge.utils.PageListUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * [业务--知识分类表]业务服务类
 */
@Service
@Slf4j
public class KnowledgeCategoryServiceImpl extends SuperServiceImpl<KnowledgeCategoryMapper, KnowledgeCategory> implements BaseService<KnowledgeCategory> {

    @Resource
    private KnowledgeCategoryMapper knowledgeCategoryMapper;

    @Autowired
    private KnowledgeMapper knowledgeMapper;

    @Autowired
    private UserService userService;

    @Override
    public BaseDao<KnowledgeCategory> getRepository() {
        return knowledgeCategoryMapper;
    }

    // 根据id查询其下级分类信息(懒加载)
    public KnowledgeCategory findAllByCategoryId(String categoryId) {
        QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PK_ID", categoryId);
        KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(categoryId);
        Boolean hasChild = findHasChirld(knowledgeCategory);
        knowledgeCategory.setHasChild(hasChild);
        if (hasChild) {
            List<KnowledgeCategory> allChildren = findAllChildren(knowledgeCategory);
            knowledgeCategory.setChildren(allChildren);
            allChildren.forEach(e -> {
                Boolean hasChirld = findHasChirld(e);
                e.setHasChild(hasChirld);
            });
        } else {
            knowledgeCategory.setChildren(findAllChildren(knowledgeCategory));
        }

        return knowledgeCategory;
    }

    // 根据id查询其下级分类信息(非懒加载)
    @Cacheable(value = "knowledgeCategory",key = "'categoryId_' + #categoryId",cacheManager = "redisCacheManager")
    public KnowledgeCategoryAdminVo getAllByCategoryId(String categoryId) {
        QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PK_ID", categoryId);
        KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(categoryId);
        KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = new KnowledgeCategoryAdminVo();
        knowledgeCategoryAdminVo.setLevel(1);
        if (knowledgeCategory != null) {
            BeanUtils.copyProperties(knowledgeCategory, knowledgeCategoryAdminVo);
        }
        List<KnowledgeCategoryAdminVo> childrens = findChildrens(knowledgeCategoryAdminVo,null);
        knowledgeCategoryAdminVo.setChildren(childrens);
        return knowledgeCategoryAdminVo;
    }

    /**
     * 知识分类列表(可带条件，条件为知识分类名称)
     *
     * @param filter
     * @return
     */
    public IPage findAllKnowledgeCategory(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        String userId = userService.queryLoginUser();
        //filter.getParams().put("userId",userId);
        IPage page = knowledgeCategoryMapper.findAllKnowledgeCategory(filter.getPage(), filter.getParams(), params);
        List<KnowledgeCategoryAdminVo> knowledgeCategoryList = page.getRecords();
        // 如果有name参数,特殊处理
        if (filter.getParams().containsKey("name")) {
            // 孩子节点集合
            List<String> childrenList = new ArrayList<>();
            // 查询其孩子节点
            knowledgeCategoryList.forEach(e -> {
                e.setLevel(1);
                e.setIsPermission(permission(e.getCreateBy(),e.getMaintainer(),userId));
                e.setOperatorName(userService.queryUser(e.getUpdateBy()).getFullName());
                e.setCreateByName(userService.queryUser(e.getCreateBy()).getFullName());
                if (StringUtils.isNotEmpty(e.getMaintainer())){
                    String maintainerName = Arrays.stream(e.getMaintainer().split(",")).
                            map(maintainer -> userService.queryOsUserDto(maintainer).getFullName()).
                            collect(Collectors.joining(","));
                    e.setMaintainerName(maintainerName);
                }
                List<KnowledgeCategoryAdminVo> childrens = findChildrens(e,userId);
                if (childrens.size() == 0) {
                    e.setChildren(null);
                } else {
                    e.setChildren(childrens);
                    childrens.forEach(h -> {
                        //h.setOperatorName(userService.queryUser(h.getUpdateBy()).getFullName());
                        //h.setCreateByName(userService.queryUser(h.getCreateBy()).getFullName());
                        //h.setMaintainerName(userService.queryUser(h.getMaintainer()).getFullName());
                        // 把孩子节点放入集合中
                        childrenList.add(h.getPkId());
                    });
                }
            });
            // 判断是否在孩子节点中(在 从查询结果中移除)
            List<KnowledgeCategoryAdminVo> result = knowledgeCategoryList.stream().filter(e -> !childrenList.contains(e.getPkId())).collect(Collectors.toList());
            // 把数据放回到分页中，结果返回
            page = PageListUtils.getPages(filter.getPage().getCurrent(), filter.getPage().getSize(), result);
        } else {
            knowledgeCategoryList.forEach(e -> {
                e.setLevel(1);
                e.setIsPermission(permission(e.getCreateBy(),e.getMaintainer(),userId));
                e.setOperatorName(userService.queryUser(e.getUpdateBy()).getFullName());
                e.setCreateByName(userService.queryUser(e.getCreateBy()).getFullName());
                if (StringUtils.isNotEmpty(e.getMaintainer())){
                    String maintainerName = Arrays.stream(e.getMaintainer().split(",")).
                            map(maintainer -> userService.queryOsUserDto(maintainer).getFullName()).
                            collect(Collectors.joining(","));
                    e.setMaintainerName(maintainerName);
                }
                List<KnowledgeCategoryAdminVo> childrens = findChildrens(e,userId);
                if (childrens.size() == 0) {
                    e.setChildren(null);
                } else {
                    e.setChildren(childrens);
                    //childrens.forEach(h -> {
                    //    h.setOperatorName(userService.queryUser(h.getUpdateBy()).getFullName());
                    //    h.setCreateByName(userService.queryUser(h.getCreateBy()).getFullName());
                    //    h.setMaintainerName(userService.queryUser(h.getMaintainer()).getFullName());
                    //});
                }
            });
            page.setRecords(knowledgeCategoryList);
        }
        return page;
    }

    /**
     * 根据categoryId查询其父类信息(递归到祖先节点 1级分级)
     *
     * @param categoryId
     * @return
     */
    public KnowledgeCategoryAdminVo findAllParentByCategoryId(String categoryId) {
        KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryMapper.selectByCategoryId(categoryId);
        // 如果父id等于0，则直接返回(0为顶级分类)
        if (!Objects.isNull(knowledgeCategoryAdminVo)) {
            if ("0".equals(knowledgeCategoryAdminVo.getParent())) {
                return knowledgeCategoryAdminVo;
            }
            KnowledgeCategoryAdminVo allParent = findAllParent(knowledgeCategoryAdminVo.getParent());
            knowledgeCategoryAdminVo.setParents(allParent);
        }
        return knowledgeCategoryAdminVo;
    }

    /**
     * 新增知识分类
     *
     * @param entity
     * @return
     */
    @Override
    @CacheEvict(value = {"knowledgeCategory","knowledgeCategoryPc"}, allEntries=true,cacheManager = "redisCacheManager")
    public int insert(KnowledgeCategory entity) {
        if (BeanUtil.isEmpty(entity.getPkId())) {
            entity.setPkId(IdGenerator.getIdStr());
        }
        if (entity.getPkId().equals(entity.getParent())){
            throw new RuntimeException("添加失败,上级无法选择自己");
        }
        // 没有parent默认为一级分类
        if (entity.getParent() == null) {
            entity.setParent("0");
        }
        // 没有sort默认为0
        if (entity.getSort() == null) {
            entity.setSort(0);
        }
        entity.setIsSys(0);
        knowledgeCategoryMapper.insertOrUpdateBySort(entity.getParent(), entity.getSort());
        entity.setUpdateBy(userService.queryLoginUser());
        return this.getRepository().insert(entity);
    }

    /**
     * 修改知识分类
     *
     * @param entity
     * @return
     */
    @Override
    @CacheEvict(value = {"knowledgeCategory","knowledgeCategoryPc"}, allEntries=true,cacheManager = "redisCacheManager")
    public int update(KnowledgeCategory entity) {
        KnowledgeCategory knowledgeCategory = this.get(entity.getPkId());
        //判断是否能进行修改(创建人或维护人满足一个即可修改)
        String userId = userService.queryLoginUser();
        if (!userId.equals(knowledgeCategory.getCreateBy())){
            throw new RuntimeException("您没有权限进行此操作");
        }
        if (StringUtils.isNotEmpty(knowledgeCategory.getMaintainer())){
            if (!Arrays.asList(knowledgeCategory.getMaintainer().split(",")).contains(userId)){
                throw new RuntimeException("您没有权限进行此操作");
            }
        }
        if (knowledgeCategory.getIsSys() == 1) {
            throw new RuntimeException("系统内置节点,无法修改");
        }
        if (entity.getPkId().equals(entity.getParent())){
            throw new RuntimeException("修改失败,上级无法选择自己");
        }
        if (entity.getParent() == null) {
            entity.setParent("0");
        }
        if (entity.getParent() != null) {
            if (entity.getSort() == null) {
                entity.setSort(0);
            }
        }
        //同一分支修改
        //1.1 已当前修改记录为父级的所有子级id
        List<String> childrenIdList = knowledgeCategoryMapper.selectAllChildren(knowledgeCategory.getPkId());
        //1.2 如果所有子级id中包含传入的父级(同级向下修改)
        if(childrenIdList.contains(entity.getParent())){
            //2.1 记录下没修改前的原父级
            String parent = knowledgeCategory.getParent();
            //2.2 把要修改的记录的直属下级的父级变成我的原父级
            QueryWrapper<KnowledgeCategory> wrapper = new QueryWrapper<>();
            wrapper.eq("PARENT",knowledgeCategory.getPkId());
            List<KnowledgeCategory> knowledgeCategories = this.baseMapper.selectList(wrapper);
            knowledgeCategoryMapper.updateByList(parent,knowledgeCategories.parallelStream().map(KnowledgeCategory::getPkId).collect(Collectors.toList()));
        }
        //变更sort
        knowledgeCategoryMapper.insertOrUpdateBySort(entity.getParent(), entity.getSort());
        entity.setIsSys(0);
        return this.getRepository().updateById(entity);
    }

    /**
     * 根据id进行数据回显
     *
     * @param pkId
     * @return
     */
    public KnowledgeCategoryAdminVo findByCategoryId(String pkId) {
        KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryMapper.selectByCategoryId(pkId);
        KnowledgeCategoryAdminVo allParent = findAllParent(knowledgeCategoryAdminVo.getParent());
        knowledgeCategoryAdminVo.setParents(allParent);
        return knowledgeCategoryAdminVo;
    }

    /**
     * 删除知识分类(若分类下还有关联的知识，无法删除)
     *
     * @param knowledgeCategoryId
     */
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = {"knowledgeCategory","knowledgeCategoryPc"}, allEntries=true,cacheManager = "redisCacheManager")
    public Integer deleteKnowledgeCategory(String knowledgeCategoryId) {
        KnowledgeCategory knowledgeCategory = this.baseMapper.selectById(knowledgeCategoryId);
        //判断是否能进行修改(创建人或维护人满足一个即可修改)
        String userId = userService.queryLoginUser();
        if (!userId.equals(knowledgeCategory.getCreateBy())){
            throw new RuntimeException("您没有权限进行此操作");
        }
        if (StringUtils.isNotEmpty(knowledgeCategory.getMaintainer())){
            if (!Arrays.asList(knowledgeCategory.getMaintainer().split(",")).contains(userId)){
                throw new RuntimeException("您没有权限进行此操作");
            }
        }
        // 查询该分类下是否还有知识
        Boolean hasKnowledge = hasKnowledge(knowledgeCategoryId);
        if (hasKnowledge) {
            return 0;
        }
        // 查询该分类的下是否还有子分类
        Boolean hasChirld = findHasChirld(knowledgeCategory);
        if (hasChirld) {
            return 1;
        }
        if (knowledgeCategory.getIsSys() == 1) {
            throw new RuntimeException("系统内置节点,无法修改");
        }
        // 变更sort
        knowledgeCategoryMapper.deleteBySort(knowledgeCategory.getParent(), knowledgeCategory.getSort());
        knowledgeCategoryMapper.deleteKnowledgeCategory(knowledgeCategoryId);
        return 2;
    }


    // 查询孩子节点(非递归)
    private List<KnowledgeCategory> findAllChildren(KnowledgeCategory knowledgeCategory) {
        QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PARENT", knowledgeCategory.getPkId());
        queryWrapper.orderByAsc("SORT");
        List<KnowledgeCategory> knowledgeCategories = knowledgeCategoryMapper.selectList(queryWrapper);
        return knowledgeCategories;
    }

    // 查询孩子节点(递归)
    public List<KnowledgeCategoryAdminVo> findChildrens(KnowledgeCategoryAdminVo knowledgeCategoryAdminVo,String userId) {
        List<KnowledgeCategoryAdminVo> knowledgeCategorieChild = knowledgeCategoryMapper.selectKnowledgeCategoryByParent(knowledgeCategoryAdminVo.getPkId());
        List<KnowledgeCategoryAdminVo> knowledgeCategories = knowledgeCategorieChild.stream().
                sorted(Comparator.comparingInt(KnowledgeCategoryAdminVo::getSort)).
                map(e -> {
                    e.setIsPermission(permission(e.getCreateBy(),e.getMaintainer(),userId));
                    e.setOperatorName(userService.queryUser(e.getUpdateBy()).getFullName());
                    e.setCreateByName(userService.queryUser(e.getCreateBy()).getFullName());
                    if (StringUtils.isNotEmpty(e.getMaintainer())){
                        String maintainerName = Arrays.stream(e.getMaintainer().split(",")).
                                map(maintainer -> userService.queryOsUserDto(maintainer).getFullName()).
                                collect(Collectors.joining(","));
                        e.setMaintainerName(maintainerName);
                    }
                    return e;
                }).
                collect(Collectors.toList());
        knowledgeCategoryAdminVo.setChildren(knowledgeCategories);
        knowledgeCategories.forEach(e -> {
            e.setLevel(knowledgeCategoryAdminVo.getLevel() + 1);
            List<KnowledgeCategoryAdminVo> allChildren = findChildrens(e,userId);
            if (allChildren.size() == 0) {
                e.setChildren(null);
            } else {
                e.setChildren(allChildren);
            }
        });
        return knowledgeCategories;
    }

    // 查询父亲节点(递归)
    private KnowledgeCategoryAdminVo findAllParent(String parentId) {
        KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryMapper.selectByCategoryId(parentId);
        if (knowledgeCategoryAdminVo != null && !("0".equals(knowledgeCategoryAdminVo.getParent()))) {
            KnowledgeCategoryAdminVo allParent = findAllParent(knowledgeCategoryAdminVo.getParent());
            knowledgeCategoryAdminVo.setParents(allParent);
        }
        return knowledgeCategoryAdminVo;
    }

    // 判断是否有孩子节点
    private Boolean findHasChirld(KnowledgeCategory knowledgeCategory) {
        QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PARENT", knowledgeCategory.getPkId());
        Integer count = knowledgeCategoryMapper.selectCount(queryWrapper);
        if (count > 0) {
            return true;
        } else {
            return false;
        }
    }

    // 判断分类下是否还有知识
    private Boolean hasKnowledge(String knowledgeCategoryId) {
        QueryWrapper<Knowledge> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("CATEGORY_ID", knowledgeCategoryId);
        List<Knowledge> knowledges = knowledgeMapper.selectList(queryWrapper);
        return knowledges.size() > 0;
    }

    /**
     * 状态为true,上升;为false,下降
     *
     * @param knowledgeCategoryId
     * @param status
     */
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = {"knowledgeCategory","knowledgeCategoryPc"}, allEntries=true,cacheManager = "redisCacheManager")
    public void goUpOrDownUp(String knowledgeCategoryId, Boolean status) {
        String userId = userService.queryLoginUser();
        KnowledgeCategory knowledgeCategory = this.get(knowledgeCategoryId);
        Integer sort = knowledgeCategory.getSort();
        String parent = knowledgeCategory.getParent();
        if (status) {
            // 上升
            QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("PARENT", parent);
            queryWrapper.orderByAsc("SORT");
            List<KnowledgeCategory> knowledgeCategoryList = knowledgeCategoryMapper.selectList(queryWrapper);
            //List<KnowledgeCategory> knowledgeCategoryList = knowledgeCategoryMapper.visibleKnowledgeCategory(parent, userId);
            List<KnowledgeCategory> collect = knowledgeCategoryList.stream().filter(e -> e.getSort() < sort).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(collect)){
                KnowledgeCategory knowledgeCategoryPre = collect.get(collect.size() - 1);
                // 交换位置
                int temp = sort;
                knowledgeCategoryMapper.swapSort(knowledgeCategoryId, knowledgeCategoryPre.getSort());
                knowledgeCategoryMapper.swapSort(knowledgeCategoryPre.getPkId(), temp);
            }
        } else {
            // 下降
            QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("PARENT", parent);
            queryWrapper.orderByAsc("SORT");
            List<KnowledgeCategory> knowledgeCategoryList = knowledgeCategoryMapper.selectList(queryWrapper);
            //List<KnowledgeCategory> knowledgeCategoryList = knowledgeCategoryMapper.visibleKnowledgeCategory(parent, userId);
            List<KnowledgeCategory> collect = knowledgeCategoryList.stream().filter(e -> e.getSort() > sort).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(collect)) {
                KnowledgeCategory knowledgeCategoryNext = collect.get(0);
                // 交换位置
                int temp = sort;
                knowledgeCategoryMapper.swapSort(knowledgeCategoryId, knowledgeCategoryNext.getSort());
                knowledgeCategoryMapper.swapSort(knowledgeCategoryNext.getPkId(), temp);
            }
        }
    }

    /**
     * PC端展示(无分页,数据直接返回)
     *
     * @param
     * @return
     */
    @Cacheable(value = "knowledgeCategoryPc",cacheManager = "redisCacheManager")
    public List<KnowledgeCategoryAdminVo> getAllKnowledgeCategory() {
        List<KnowledgeCategoryAdminVo> knowledgeCategoryList = knowledgeCategoryMapper.getAllKnowledgeCategory();
        // 查询顶级记录的所有子集合(顶级记录id为0)
        List<KnowledgeCategoryAdminVo> collect = knowledgeCategoryList.
                stream().
                filter(knowledgeCategoryAdminVo -> "0".equals(knowledgeCategoryAdminVo.getParent())).
                sorted(Comparator.comparingInt(KnowledgeCategoryAdminVo::getSort)).
                collect(Collectors.toList());
        // 查询其孩子节点
        collect.forEach(e -> {
            e.setLevel(1);
            List<KnowledgeCategoryAdminVo> childrens = findChildrens(e,null);
            e.setChildren(childrens);
        });
        return collect;
    }

    /**
     * 是否有权限 0 有 1 没有
     * @param createBy
     * @param maintainer
     * @return
     */
    private int permission(String createBy,String maintainer,String userId){
        if (StringUtils.isNotEmpty(userId)){
            if (userId.equals(createBy) || (StringUtils.isNotEmpty(maintainer) && Arrays.asList(maintainer.split(",")).contains(userId))){
                return 0;
            } else {
                return 1;
            }
        } else {
            return 2;
        }
    }
}
