package com.redxun.search.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.redxun.common.base.entity.IUser;
import com.redxun.common.tool.IdGenerator;
import com.redxun.common.utils.ContextUtil;
import com.redxun.knowledge.analysis.entity.dao.SearchParticipleLog;
import com.redxun.knowledge.analysis.mapper.SearchParticipleLogMapper;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.mapper.KnowledgeMapper;
import com.redxun.search.service.SearchService;
import com.redxun.search.utils.HttpRequestHelper;
import com.redxun.search.utils.MD5;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 文件名: SearchServiceImpl
 * 作者: Zizg
 * 时间: 2022/12/13
 * 描述:
 * 修改人:
 * 修改时间:
 * 修改内容:
 */
@Slf4j
@Service
@RefreshScope
public class SearchServiceImpl implements SearchService {

    @Value("${search.url}")
    private String searchUrl; //测试环境搜索服务地址

    @Autowired
    private UserService userService;

    @Autowired
    private KnowledgeMapper knowledgeMapper;

    @Autowired
    private SearchParticipleLogMapper searchParticipleLogMapper;

    @SneakyThrows
    @Override
    public Map<String, Object> search(String keyword, Integer pageIndex, Integer pageSize,
                                      Integer range, Integer secretType, Integer orderFiled, Integer orderSort, HttpServletRequest httpServletRequest) {
        Map<String, Object> result = new HashMap<>();
        Map<String, Object> paramMap = new HashMap<>();
        Map<String, Object> headerMap = new HashMap<>();
        log.info("请求参数keyword -> {}",keyword);
        paramMap.put("keyword", URLEncoder.encode(keyword, "utf-8"));
        paramMap.put("pageIndex", pageIndex);
        paramMap.put("pageSize", pageSize);
        paramMap.put("range", range);
        paramMap.put("secretType", secretType);
        if (orderFiled != null) {
            paramMap.put("orderFiled", orderFiled);
        }
        if (orderSort != null) {
            paramMap.put("orderSort", orderSort);
        }
        // 对signature进行加密
        Map<String, Object> params = getParams(httpServletRequest);
        String sign = makeSign(params, "uyjg4h61j5y3t59t");
        // String sign = "uyjg4h61j5y3t59t";
        headerMap.put("signature", sign);
        headerMap.put("businessKey", httpServletRequest.getHeader("businessKey"));
        String userId = userService.queryLoginUser();
        headerMap.put("userId", userId);
        //发送搜索请求
        JSONObject respone = HttpRequestHelper.sendRequest(headerMap, paramMap, searchUrl + "/api-kmsearch/search/search");
        if (null != respone && 200 == respone.getIntValue("code")) {
            result.put("code", respone.getIntValue("code"));
            result.put("message", respone.getString("message"));
            result.put("data", respone.getJSONObject("data"));
        } else {
            assert respone != null;
            result.put("code", respone.getIntValue("code"));
            result.put("message", respone.getString("message"));
            result.put("data", respone.getJSONObject("data"));
        }
        return result;
    }

    @SneakyThrows
    @Override
    public Map<String, Object> notice(String pkId, String documentId, String documentTitle, String documentName, String documentPath,
                                      String documentUrl, String downloadUrl, String documentMd5, String uploaderName,
                                      String uploader, String uploaderHead, String remark, Integer needText) {
        Map<String, Object> result = new HashMap<>();
        Map<String, Object> paramMap = new HashMap<>();
        Map<String, Object> headerMap = new HashMap<>();
        if (documentId != null) {
            paramMap.put("documentId", documentId);
        }
        paramMap.put("documentTitle", documentTitle);
        paramMap.put("documentName", documentName);
        paramMap.put("documentPath", documentPath);
        paramMap.put("documentUrl", documentUrl);
        paramMap.put("downloadUrl", downloadUrl);
        paramMap.put("documentMd5", documentMd5);
        paramMap.put("uploaderName", uploaderName);
        paramMap.put("uploader", uploader);
        paramMap.put("uploaderHead", uploaderHead);
        paramMap.put("remark", remark);
        paramMap.put("needText", needText);
        paramMap.entrySet().removeIf(e -> e.getValue() == null);
        // 对signature进行加密
        String sign = makeSign(paramMap, "uyjg4h61j5y3t59t");
        // String sign = "uyjg4h61j5y3t59t";
        headerMap.put("signature", sign);
        headerMap.put("businessKey", "knowlege-manager");
        headerMap.put("userId", userService.queryLoginUser());
        JSONObject respone = HttpRequestHelper.sendRequest(headerMap, paramsEncoder(paramMap), searchUrl + "/api-kmsearch/search/notice");
        if (null != respone && 200 == respone.getIntValue("code")) {
            result.put("code", respone.getIntValue("code"));
            result.put("message", respone.getString("message"));
            // 上传成功,文件Id回填
            result.put("data", respone.getString("data"));
            Knowledge knowledge = knowledgeMapper.selectById(pkId);
            knowledge.setPkId(knowledge.getPkId());
            knowledge.setDocumentId((String) result.get("data"));
            knowledgeMapper.updateById(knowledge);

        } else {
            assert respone != null;
            result.put("code", respone.getIntValue("code"));
            result.put("message", respone.getString("message"));
            result.put("data", respone.getJSONObject("data"));
        }
        return result;
    }

    @Override
    public Map<String, Object> view(String documentId, String userId, String secretType, HttpServletRequest httpServletRequest) {
        Map<String, Object> result = new HashMap<>();
        Map<String, Object> paramMap = new HashMap<>();
        Map<String, Object> headerMap = new HashMap<>();
        paramMap.put("documentId", documentId);
        paramMap.put("userId", userId);
        paramMap.put("secretType", secretType);
        // 对signature进行加密
        Map<String, Object> params = getParams(httpServletRequest);
        String sign = makeSign(params, "uyjg4h61j5y3t59t");
        // String sign = "uyjg4h61j5y3t59t";
        headerMap.put("signature", sign);
        headerMap.put("businessKey", httpServletRequest.getHeader("businessKey"));
        headerMap.put("userId", userService.queryLoginUser());
        JSONObject respone = HttpRequestHelper.sendRequest(headerMap, paramMap, searchUrl + "/api-kmsearch/search/view");
        if (null != respone && 200 == respone.getIntValue("code")) {
            result.put("code", respone.getIntValue("code"));
            result.put("message", respone.getString("message"));
            result.put("data", respone.getJSONObject("data"));
        } else {
            assert respone != null;
            result.put("code", respone.getIntValue("code"));
            result.put("message", respone.getString("message"));
            result.put("data", respone.getJSONObject("data"));
        }
        return result;
    }

    @Override
    public Map<String, Object> remove(String documentId) {
        Map<String, Object> result = new HashMap<>();
        Map<String, Object> paramMap = new HashMap<>();
        Map<String, Object> headerMap = new HashMap<>();
        // 传入文件Id
        paramMap.put("documentId", documentId);
        // 对signature进行加密
        String sign = makeSign(paramMap, "uyjg4h61j5y3t59t");
        // String sign = "uyjg4h61j5y3t59t";
        headerMap.put("signature", sign);
        headerMap.put("businessKey", "knowlege-manager");
        headerMap.put("userId", userService.queryLoginUser());
        JSONObject respone = HttpRequestHelper.sendRequest(headerMap, paramMap, searchUrl + "/api-kmsearch/search/remove");
        if (null != respone && 200 == respone.getIntValue("code")) {
            result.put("code", respone.getIntValue("code"));
            result.put("message", respone.getString("message"));
            result.put("data", respone.getString("data"));
        } else {
            assert respone != null;
            result.put("code", respone.getIntValue("code"));
            result.put("message", respone.getString("message"));
            result.put("data", respone.getJSONObject("data"));
        }
        return result;
    }

    /**
     * 获取请求参数
     *
     * @param request
     * @return
     */
    public Map<String, Object> getParams(HttpServletRequest request) {// 未考虑数组排序之后逗号连接
        Map<String, String[]> map = request.getParameterMap();
        SortedMap<String, Object> result = new TreeMap<>();
        for (Map.Entry<String, String[]> entry : map.entrySet()) {
            if (entry.getKey().equalsIgnoreCase("file")) {
                continue;
            }
            String[] value = entry.getValue();
            if (value != null && value.length > 0) {
                result.put(entry.getKey(), Stream.of(value).sorted().collect(Collectors.joining(",")));
            } else {
                result.put(entry.getKey(), null);
            }
        }
        return result;
    }

    /**
     * 制作签名
     *
     * @param params
     * @param secret
     * @return
     */
    public String makeSign(Map<String, Object> params, String secret) {
        List<String> keys = new ArrayList<>(params.keySet());
        Collections.sort(keys);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            sb.append(key);
            // sb.append("=");
            sb.append(params.get(key));
            // sb.append("&");
        }
        // if(sb.length()>0) {
        // sb.deleteCharAt(sb.length()-1);
        // }
        sb.append(secret);
        log.info("加密后的secret -> {}", MD5.encrypt(sb.toString()).toLowerCase());
        return MD5.encrypt(sb.toString()).toLowerCase();
    }

    /**
     * 参数编码(httpclient处理发送特殊字符)
     *
     * @param params
     * @return
     */
    private Map<String, Object> paramsEncoder(Map<String, Object> params) throws UnsupportedEncodingException {
        log.info("请求参数 -> {}", params);
        Map<String, Object> result = new HashMap<>();
        Set<String> keySet = params.keySet();
        for (String key : keySet) {
            Object value = params.get(key);
            value = URLEncoder.encode(value.toString(), "utf-8");
            result.put(key, value);
        }
        return result;
    }

    public void keywordParticiple(String keyword, IUser users) {
        try {
            Map<String, Object> headerMap = new HashMap<>();
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("keyword", keyword);
            // 对signature进行加密
            String sign = makeSign(paramMap, "uyjg4h61j5y3t59t");
            headerMap.put("signature", sign);
            headerMap.put("businessKey", "knowlege-manager");
            headerMap.put("userId", users.getUserId());
            JSONObject respone = HttpRequestHelper.sendRequest(headerMap, paramsEncoder(paramMap), searchUrl + "/api-kmsearch/search/analyze");
            if (null != respone && 200 == respone.getIntValue("code")) {
                Object[] data = respone.getJSONArray("data").toArray();
                log.info("分词后数据为 -> {}", data);
                Arrays.stream(data).forEach(e -> {
                    //存入高频词分词表
                    SearchParticipleLog searchParticipleLog = new SearchParticipleLog();
                    searchParticipleLog.setPkId(IdGenerator.getIdStr());
                    searchParticipleLog.setParticiple(e.toString());
                    searchParticipleLog.setUserId(users.getUserId());
                    searchParticipleLog.setCreateDepId(users.getDeptId());
                    searchParticipleLog.setTenantId(users.getTenantId());
                    searchParticipleLog.setCompanyId(users.getCompanyId());
                    searchParticipleLog.setCreateBy(users.getUserId());
                    searchParticipleLog.setCreateTime(new Date());
                    searchParticipleLog.setUpdateBy(users.getUserId());
                    searchParticipleLog.setUpdateTime(new Date());
                    searchParticipleLogMapper.insert(searchParticipleLog);
                });
            } else {
                throw new Exception("未知错误");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
